<?php

namespace Drupal\entity_preprocess_services;

use Drupal\Core\Entity\EntityInterface;
use Drupal\entity_preprocess_services\PreprocessService\PreprocessServiceInterface;
use Symfony\Component\DependencyInjection\ContainerAwareTrait;

/**
 * Class EntityPreprocessServicesManager.
 *
 * @package Drupal\entity_preprocess_services
 */
class EntityPreprocessServicesManager {

  use ContainerAwareTrait;

  /**
   * All entity preprocess services.
   *
   * @var array
   *   An array containing the preprocess services.
   */
  protected $entityPreprocessServices = [];

  /**
   * Adds a entity preprocess service definition to our array of services.
   *
   * @param array $serviceDefinition
   *   The service definition.
   */
  public function addEntityPreprocessService(array $serviceDefinition) {
    $this->entityPreprocessServices[] = $serviceDefinition;
  }

  /**
   * Loads the entity preprocess services.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   * @param string $viewMode
   *   The view mode to preprocess.
   *
   * @return array
   *   The services found for this entity.
   *
   * @throws \Exception
   *   When a service does not implement the correct interface.
   */
  public function getEntityPreprocessServices(EntityInterface $entity, string $viewMode) : array {
    $matchingServices = [];

    foreach ($this->entityPreprocessServices as $serviceDefinition) {
      // This service does not match the entity type.
      if ($serviceDefinition['entity_type'] !== $entity->getEntityTypeId()) {
        continue;
      }

      // This service has a bundle defined and the bundle does not match.
      // Meaning we can skip this service.
      if ($serviceDefinition['bundle'] && $serviceDefinition['bundle'] !== $entity->bundle()) {
        continue;
      }

      // This service has a view mode defined and the view mode does not match.
      // Meaning we can skip this service.
      if ($serviceDefinition['view_mode'] && $serviceDefinition['view_mode'] !== $viewMode) {
        continue;
      }

      $preprocessService = $this->loadPreprocessService($serviceDefinition['service']);

      $matchingServices[] = $preprocessService
        ->setEntity($entity)
        ->setViewMode($viewMode);
    }

    return $matchingServices;
  }

  /**
   * Loads a preprocess service by id.
   *
   * @param string $id
   *   The id of the service.
   *
   * @return \Drupal\entity_preprocess_services\PreprocessService\PreprocessServiceInterface
   *   The preprocess service interface.
   *
   * @throws \Exception
   *   When service does not implement PreprocessServiceInterface.
   */
  protected function loadPreprocessService(string $id) : PreprocessServiceInterface {
    $preprocessService = $this->container->get($id);

    if (!$preprocessService instanceof PreprocessServiceInterface) {
      throw new \Exception(
        sprintf('The preprocess service must implement %s', PreprocessServiceInterface::class)
      );
    }

    return $preprocessService;
  }

}
